// Copyright (C) 2018 Sayutin Dmitry.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 3

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; If not, see <http://www.gnu.org/licenses/>.


#include "testlib.h"
#include <stdint.h>

struct lim_t {
    int max_p;
    int max_n;
    int max_b;
};

const int max_p = 2;
const int max_n = 100 * 1000;
const int max_b = 10 * 1000;

const int max_val = 1000 * 1000 * 1000;

/*
\hline
0 & 0 & --& -- & -- & -- & Тесты из условия \\
\hline
1 & 10 & $p = 1$ & $n \le 1000$ & $b = 1$ & -- & -- \\
\hline
2 & 10 & $p = 1$ & $n \le 1000$ & ~--- & 1 & -- \\
\hline
3 & 10 & ~--- & $n \le 100$ & $b = 1$ & -- & -- \\
\hline
4 & 15 & ~--- & $n \le 100$ & $b \le 30$ & 0, 3 & -- \\
\hline
5 & 20 & ~--- & $n \le 1000$ & $b \le 30$ & 0, 1, 3, 4 & -- \\
\hline
6 & 10 & $p = 1$ & ~--- & ~--- & 1, 2 & \textbf{Offline-проверка} \\
\hline
7 & 25 & ~--- & ~--- & ~--- & 0--6 & \textbf{Offline-проверка} \\
\hline
*/

lim_t groups[8] = {lim_t {max_p,  max_n,  max_b},  // 0, samples.
                   lim_t {1,      1000,       1},  // 1 (1prep; matching)
                   lim_t {1,      1000,   max_b},  // 2 (1prep; slowgreedy)
                   lim_t {max_p,  100,        1},  // 3 (2preps; matching)
                   lim_t {max_p,  100,       30},  // 4 (2preps; slow search + slow greedy)
                   lim_t {max_p,  1000,      30},  // 5 (2preps; slow search + fastgreedy)
                   lim_t {1,      max_n,  max_b},  // 6 (1prep; fastgreedy).
                   lim_t {max_p,  max_n,  max_b}}; // 7 (2preps; greedy; BS on split; fast sim).

int main(int argc, char** argv) {
    registerValidation(argc, argv);

    lim_t lim;
    if (validator.group() == "")
        lim = groups[0];
    else if (not (validator.group().size() == 1 and '0' <= validator.group()[0] and validator.group()[0] <= '7'))
        return 1;
    else
        lim = groups[validator.group()[0] - '0'];
    
    inf.readInt(1, lim.max_p, "p");
    inf.readSpace();

    int n = inf.readInt(2, lim.max_n, "n");
    inf.readSpace();

    inf.readInt(1, n - 1, "d");
    inf.readSpace();

    int B = inf.readInt(1, lim.max_b, "b");
    inf.readEoln();

    int64_t sum = 0;
    
    for (int i = 0; i != n; ++i) {
        sum += inf.readInt(0, max_val, "a[" + vtos(i) + "]");

        if (i != n - 1)
            inf.readSpace();
        else
            inf.readEoln();
    }

    ensuref(sum == n * int64_t(B), "sum a_i = %lld, n * B = %lld", sum, n * int64_t(B));
    inf.readEof();
    
    return 0;
}
