// Copyright (C) 2017 Sayutin Dmitry.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 3
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; If not, see <http://www.gnu.org/licenses/>.

#include <iostream>
#include <vector>
#include <stdint.h>
#include <algorithm>
#include <set>
#include <map>
#include <array>
#include <queue>
#include <stack>
#include <functional>
#include <utility>
#include <string>
#include <assert.h>
#include <iterator>
#include <cstdint>
#include <cinttypes>
#include <string.h>
#include <random>
#include <numeric>
#include <tuple>

using std::cin;
using std::cout;
using std::cerr;

using std::vector;
using std::map;
using std::array;
using std::set;
using std::string;

using std::pair;
using std::make_pair;

using std::min;
using std::abs;
using std::max;

using std::unique;
using std::sort;
using std::generate;
using std::reverse;
using std::min_element;
using std::max_element;

#ifdef LOCAL
#define LASSERT(X) assert(X)
#else
#define LASSERT(X) {}
#endif

template <typename T>
T input() {
    T res;
    cin >> res;
    LASSERT(cin);
    return res;
}

template <typename IT>
void input_seq(IT b, IT e) {
    std::generate(b, e, input<typename std::remove_reference<decltype(*b)>::type>);
}

#define SZ(vec)         int((vec).size())
#define ALL(data)       data.begin(),data.end()
#define RALL(data)      data.rbegin(),data.rend()
#define TYPEMAX(type)   std::numeric_limits<type>::max()
#define TYPEMIN(type)   std::numeric_limits<type>::min()

#define pb push_back
#define eb emplace_back

bool dfs(const vector<pair<int, int>>& graph, vector<int>& rpair, vector<char>& used, int v) {
    used[v] = true;
    
    for (int u = graph[v].first; u != graph[v].second; ++u)
        if (rpair[u] == -1 or (not used[rpair[u]] and dfs(graph, rpair, used, rpair[u]))) {
            rpair[u] = v;
            return true;
        }
    
    return false;
}

int main() {
    std::iostream::sync_with_stdio(false);
    cin.tie(nullptr);
    cout.tie(nullptr);
    
    // code here.
    int p, n, d, B;
    cin >> p >> n >> d >> B;

    if (B != 1)
        return 1;

    vector<int> lst;
    for (int i = 0; i != n; ++i)
        for (int t = input<int>(); t != 0; --t)
            lst.push_back(i);

    auto score = [&](int l, int r, int lst_l, int lst_r, bool lft) {
        vector<int> rpair(lst_r - lst_l + 1, -1);

        vector<pair<int, int>> graph(r - l + 1);
        for (int i = l; i <= r; ++i) {
            int minl, minr;

            if (lft) {
                minl = l;
                minr = (int)min((int64_t)(n - 1), i + (i - l + 1) * int64_t(d));
            } else {
                minr = r;
                minl = (int)max((int64_t)0, i - (r - i + 1) * int64_t(d));
            }
            
            graph[i - l].first  = std::lower_bound(lst.begin() + lst_l, lst.begin() + (lst_r + 1), minl) - lst.begin() - lst_l;
            graph[i - l].second = std::upper_bound(lst.begin() + lst_l, lst.begin() + (lst_r + 1), minr) - lst.begin() - lst_l;
        }

        int matched = 0;
        for (int i = 0; i <= r - l; ++i) {
            vector<char> used(n, false);
            matched += dfs(graph, rpair, used, i);
        }

        int bad = (r - l + 1) - matched;

        return bad;
    };
    
    if (p == 1) {
        cout << score(0, n - 1, 0, SZ(lst) - 1, true) << "\n";
    } else {
        // p == 2.

        int ans = 1e9;

        int P1 = (n % 2 == 0 ? n / 2 : n / 2 + 1) - 1;
        int P2 = P1 + 1;
        
        for (int split = 0; split <= SZ(lst); ++split)
            ans = min(ans, max(score(0, P1, 0, split - 1, true), score(P2, n - 1, split, SZ(lst) - 1, false)));

        cout << ans << "\n";
    }
    return 0;
}
