// Copyright (C) 2017 Sayutin Dmitry.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 3
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; If not, see <http://www.gnu.org/licenses/>.

#include <iostream>
#include <vector>
#include <stdint.h>
#include <algorithm>
#include <set>
#include <map>
#include <array>
#include <queue>
#include <stack>
#include <functional>
#include <utility>
#include <string>
#include <assert.h>
#include <iterator>
#include <cstdint>
#include <cinttypes>
#include <string.h>
#include <random>
#include <numeric>
#include <tuple>

using std::cin;
using std::cout;
using std::cerr;

using std::vector;
using std::map;
using std::array;
using std::set;
using std::string;

using std::pair;
using std::make_pair;

using std::min;
using std::abs;
using std::max;

using std::unique;
using std::sort;
using std::generate;
using std::reverse;
using std::min_element;
using std::max_element;

#ifdef LOCAL
#define LASSERT(X) assert(X)
#else
#define LASSERT(X) {}
#endif

template <typename T>
T input() {
    T res;
    cin >> res;
    LASSERT(cin);
    return res;
}

template <typename IT>
void input_seq(IT b, IT e) {
    std::generate(b, e, input<typename std::remove_reference<decltype(*b)>::type>);
}

#define SZ(vec)         int((vec).size())
#define ALL(data)       data.begin(),data.end()
#define RALL(data)      data.rbegin(),data.rend()
#define TYPEMAX(type)   std::numeric_limits<type>::max()
#define TYPEMIN(type)   std::numeric_limits<type>::min()

#define pb push_back
#define eb emplace_back

int sign(int v) {
    if (v < 0)
        return -1;
    else if (v == 0)
        return 0;
    else
        return +1;
}

int main() {
    std::iostream::sync_with_stdio(false);
    cin.tie(nullptr);
    cout.tie(nullptr);
    
    // code here.
    int p, n, d, B;
    cin >> p >> n >> d >> B;
    
    vector<int64_t> a(n);
    for (int i = 0; i != n; ++i)
        a[i] = input<int>();

    vector<int64_t> p0(n + 1);
    for (int i = 0; i != n; ++i)
        p0[i + 1] = p0[i] + a[i];

    auto score = [&](int l, int r, int64_t lst_l, int64_t lst_r, bool lft) {
        vector<int64_t> a(n);
        for (int i = 0; i != n; ++i) {
            int64_t sl = p0[i];
            int64_t sr = p0[i + 1] - 1;

            a[lft ? i : n - 1 - i] = max((int64_t)0, min(sr, lst_r) - max(sl, lst_l) + 1);
        }
                
        if (not lft) {
            l = n - 1 - l;
            r = n - 1 - r;
            std::swap(l, r);
        }

        vector<int64_t> psums(n + 1);
        for (int i = 0; i != n; ++i)
            psums[i + 1] = psums[i] + a[i];

        int64_t debt = 0;
        for (int i = l; i <= r; ++i) {
            if (debt > 0) {
                int64_t dlt = min(a[i], debt);
                a[i] -= dlt;
                debt -= dlt;
            }
            if (a[i] < B) {
                if (psums[min((int64_t)n, i + int64_t(d) * (i + 1) + 1)] - psums[i + 1] + a[i] - debt >= B) {
                    debt += B - a[i];
                    a[i] = B;
                } else if (i != r) {
                    a[i + 1] += a[i];
                    a[i] = 0;
                } else {
                    int64_t dlt = psums[min((int64_t)n, i + int64_t(d) * (i + 1) + 1)] - psums[i + 1] - debt;
                    a[i] += dlt;
                    debt += dlt;
                }
            }

            if (a[i] > B and i != r) {
                a[i + 1] += a[i] - B;
                a[i] = B;
            }
        }

        for (int i = r + 1; i != n and debt > 0; ++i) {
            int64_t dlt = min(debt, a[i]);
            a[i] -= dlt;
            debt -= dlt;
        }

        int bad = 0;
        for (int i = l; i <= r; ++i)
            if (a[i] < B)
                ++bad;

        return bad;
    };
    
    if (p == 1) {
        cout << score(0, n - 1, 0, p0.back() - 1, true) << "\n";
    } else {
        int ans = 1e9;

        int P1 = (n % 2 == 0 ? n / 2 : n / 2 + 1) - 1;
        int P2 = P1 + 1;
        
        auto bin_f = [&](int64_t val) {
            auto t1 = score(0, P1, 0, val - 1, true);
            auto t2 = score(P2, n - 1, val, p0.back() - 1, false);
            
            return sign(t1 - t2);
        };

        auto ans_f = [&](int64_t val) {
            auto t1 = score(0, P1, 0, val - 1, true);
            auto t2 = score(P2, n - 1, val, p0.back() - 1, false);

            return max(t1, t2);
        };

        int64_t L = 0;
        int64_t R = p0.back();

        // assume bin_f(L) >= 0, bin_f(R) < 0.

        if (bin_f(L) < 0)
            ans = min(ans, ans_f(L));
        else if (bin_f(R) >= 0)
            ans = min(ans, ans_f(R));
        else {
            while (R - L > 1) {
                int64_t M = L + (R - L) / 2;

                if (bin_f(M) >= 0)
                    L = M;
                else
                    R = M;
            }

            ans = min(ans, min(ans_f(L), ans_f(R)));
        }
        
        cout << ans << "\n";
    }
    return 0;
}
